@@ -72,13 +72,12 @@
 #' \url{https://pubchemdocs.ncbi.nlm.nih.gov/programmatic-access}, and the data
 #' usage policies of the indicidual data sources
 #' \url{https://pubchem.ncbi.nlm.nih.gov/sources/}.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
-#' @author Tamás Stirling, \email{stirling.tamas@@gmail.com}
 #' @import httr
 #' @importFrom purrr map map2
 #' @importFrom jsonlite fromJSON
 #' @importFrom stats rgamma
 #' @importFrom tibble enframe
+#' @importFrom utils URLencode
 #' @export
 #' @examples
 #' \donttest{
@@ -122,11 +121,14 @@ get_cid <-
            arg = NULL,
            first = NULL,
            ...) {
+
+    if (!ping_service("pc")) stop(webchem_message("service_down"))
+
   #deprecate `first`
-  if (!is.null(first) && first == TRUE) {
+  if (!is.null(first) && first) {
     message("`first = TRUE` is deprecated. Use `match = 'first'` instead")
     match <- "first"
-  } else if (!is.null(first) && first == FALSE) {
+  } else if (!is.null(first) && !first) {
     message("`first = FALSE` is deprecated. Use `match = 'all'` instead")
     match <- "all"
   }
@@ -160,7 +162,7 @@ get_cid <-
       from <- match.arg(from, choices = from_choices)
     }
     if (domain == "substance") {
-      if (grepl("^sourceid/", from) == FALSE) {
+      if (!grepl("^sourceid/", from)) {
         from <- match.arg(from, choices = c("sid", "name", xref, "sourceall"))
       }
     }
@@ -170,31 +172,47 @@ get_cid <-
     match <- match.arg(match)
     foo <- function(query, from, domain, match, verbose, arg, ...) {
       if (is.na(query)) {
-        if (verbose) message(paste0(query, " is invalid. Returning NA."))
-        return(NA)
+        if (verbose) webchem_message("na")
+        return(tibble::tibble("query" = NA, "cid" = NA))
       }
-      if (verbose) {
-        message(paste0("Querying ", query, ". "), appendLF = FALSE)
-      }
-      if (is.character(query)) query <- URLencode(query)
+      if (verbose) webchem_message("query", query, appendLF = FALSE)
       if (from %in% structure_search) {
         qurl <- paste("https://pubchem.ncbi.nlm.nih.gov/rest/pug",
-                      domain, from, query, "json", sep = "/")
-      }
-      else {
+                      domain,
+                      from,
+                      URLencode(as.character(query), reserved = TRUE),
+                      "json",
+                      sep = "/")
+      } else {
         qurl <- paste("https://pubchem.ncbi.nlm.nih.gov/rest/pug",
-                      domain, from, query, "cids", "json", sep = "/")
+                      domain,
+                      from,
+                      URLencode(as.character(query), reserved = TRUE),
+                      "cids",
+                      "json",
+                      sep = "/")
       }
       if (!is.null(arg)) qurl <- paste0(qurl, "?", arg)
       Sys.sleep(rgamma(1, shape = 15, scale = 1 / 10))
       if (from == "inchi") {
         qurl <- paste("https://pubchem.ncbi.nlm.nih.gov/rest/pug",
                       domain, from, "cids", "json", sep = "/")
-        res <- httr::POST(qurl, body = paste0("inchi=", query),
-                          user_agent("webchem"))
+        res <- try(httr::RETRY("POST",
+                               qurl,
+                               user_agent(webchem_url()),
+                               body = paste0("inchi=", query),
+                               terminate_on = 404,
+                               quiet = TRUE), silent = TRUE)
+      } else {
+        res <- try(httr::RETRY("POST",
+                               qurl,
+                               user_agent(webchem_url()),
+                               terminate_on = c(202, 404),
+                               quiet = TRUE), silent = TRUE)
       }
-      else {
-        res <- httr::POST(qurl, user_agent("webchem"))
+      if (inherits(res, "try-error")) {
+        if (verbose) webchem_message("service_down")
+        return(tibble::tibble("query" = query, "cid" = NA))
       }
       if (res$status_code != 200) {
         if (res$status_code == 202) {
@@ -204,16 +222,24 @@ get_cid <-
                         "listkey", listkey, "cids", "json", sep = "/")
           while (res$status_code == 202) {
             Sys.sleep(5 + rgamma(1, shape = 15, scale = 1 / 10))
-            res <- httr::POST(qurl, user_agent("webchem"))
+            res <- try(httr::RETRY("POST",
+                                   qurl,
+                                   user_agent(webchem_url()),
+                                   terminate_on = 404,
+                                   quiet = TRUE), silent = TRUE)
+            if (inherits(res, "try-error")) {
+              if (verbose) webchem_message("service_down")
+              return(tibble::tibble("query" = query, "cid" = NA))
+            }
           }
           if (res$status_code != 200) {
             if (verbose) message(httr::message_for_status(res))
-            return(NA)
+            return(tibble::tibble("query" = query, "cid" = NA))
           }
         }
         else{
           if (verbose) message(httr::message_for_status(res))
-          return(NA)
+          return(tibble::tibble("query" = query, "cid" = NA))
         }
       }
       if (verbose) message(httr::message_for_status(res))
@@ -228,18 +254,14 @@ get_cid <-
         cont <- jsonlite::fromJSON(cont)$InformationList$Information$CID
       }
       out <- unique(unlist(cont))
-      out <- matcher(x = out, query = query, match = match, verbose = verbose)
+      out <- matcher(x = out, query = query, match = match, from = from, verbose = verbose)
       out <- as.character(out)
-      names(out) <- NULL
-      return(out)
+      return(tibble::tibble("query" = query, "cid" = out))
     }
     out <- map(query,
              ~foo(query = .x, from = from, domain = domain, match = match,
                   verbose = verbose, arg = arg))
-    out <- setNames(out, query)
-    out <-
-    lapply(out, enframe, name = NULL, value = "cid") %>%
-    bind_rows(.id = "query")
+    out <- dplyr::bind_rows(out)
     return(out)
 }
 
@@ -253,13 +275,12 @@ get_cid <-
 #' @param properties character vector; properties to retrieve, e.g.
 #' c("MolecularFormula", "MolecularWeight"). If NULL (default) all available
 #' properties are retrieved. See
-#' \url{https://pubchem.ncbi.nlm.nih.gov/pug_rest/PUG_REST.html#_Toc409516770}
+#' \url{https://pubchemdocs.ncbi.nlm.nih.gov/pug-rest}
 #' for a list of all available properties.
 #' @param verbose logical; should a verbose output be printed to the console?
 #' @param ... currently not used.
 #'
 #' @return a data.frame
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @seealso \code{\link{get_cid}}, \code{\link{pc_sect}}
 #' @references Wang, Y., J. Xiao, T. O. Suzek, et al. 2009 PubChem: A Public
 #' Information System for
@@ -298,7 +319,13 @@ get_cid <-
 #' "CanonicalSMILES"))
 #' }
 pc_prop <- function(cid, properties = NULL, verbose = TRUE, ...) {
-  # cid <- c("5564", "7843")
+
+  if (!ping_service("pc")) stop(webchem_message("service_down"))
+
+  if (mean(is.na(cid)) == 1) {
+    if (verbose) webchem_message("na")
+    return(NA)
+  }
   napos <- which(is.na(cid))
   cid_o <- cid
   cid <- cid[!is.na(cid)]
@@ -325,44 +352,51 @@ pc_prop <- function(cid, properties = NULL, verbose = TRUE, ...) {
   output <- paste0("/property/", properties, "/JSON")
 
   qurl <- paste0(prolog, input, output)
-  if (verbose)
-    message(qurl)
+  if (verbose) webchem_message("query_all", appendLF = FALSE)
   Sys.sleep(0.2)
-  cont <- try(content(POST(qurl,
-                           body = list("cid" = paste(cid, collapse = ",")
-                                       )),
-                      type = "text", encoding = "UTF-8"),
-              silent = TRUE
-  )
-  if (inherits(cont, "try-error")) {
-    warning("Problem with web service encountered... Returning NA.")
+  res <- try(httr::RETRY("POST",
+                         qurl,
+                         httr::user_agent(webchem_url()),
+                         body = list("cid" = paste(cid, collapse = ",")),
+                         terminate_on = 404,
+                         quiet = TRUE), silent = TRUE)
+  if (inherits(res, "try-error")) {
+    if (verbose) webchem_message("service_down")
     return(NA)
   }
-  cont <- fromJSON(cont)
-  if (names(cont) == "Fault") {
-    warning(cont$Fault$Message, ". ", cont$Fault$Details, ". Returning NA.")
-    return(NA)
-  }
-  out <- cont$PropertyTable[[1]]
-  # insert NA rows
-  narow <- rep(NA, ncol(out))
-  for (i in seq_along(napos)) {
-    #capture NAs at beginning
-    firstnna <- min(which(!is.na(cid_o)))
-    if (napos[i] <  firstnna) {
-      out <- rbind(narow, out)
-    } else {
-      # capture NAs at end
-      if (napos[i] > nrow(out)) {
-        # print(napos[i])
-        out <- rbind(out, narow)
-      } else {
-        out <- rbind(out[1:(napos[i] - 1), ], narow, out[napos[i]:nrow(out), ])
+  if (verbose) message(httr::message_for_status(res))
+  if (res$status_code == 200) {
+    cont <- jsonlite::fromJSON(rawToChar(res$content))
+    if (names(cont) == "Fault") {
+      if (verbose) {
+        message(cont$Fault$Message, ". ", cont$Fault$Details, ". Returning NA.")
       }
-    }}
-  rownames(out) <- NULL
-  class(out) <- c("pc_prop", "data.frame")
-  return(out)
+      return(NA)
+    }
+    out <- cont$PropertyTable[[1]]
+    # insert NA rows
+    narow <- rep(NA, ncol(out))
+    for (i in seq_along(napos)) {
+      #capture NAs at beginning
+      firstnna <- min(which(!is.na(cid_o)))
+      if (napos[i] <  firstnna) {
+        out <- rbind(narow, out)
+      } else {
+        # capture NAs at end
+        if (napos[i] > nrow(out)) {
+          # print(napos[i])
+          out <- rbind(out, narow)
+        } else {
+          out <- rbind(out[1:(napos[i] - 1), ], narow, out[napos[i]:nrow(out), ])
+        }
+      }}
+    rownames(out) <- NULL
+    class(out) <- c("pc_prop", "data.frame")
+    return(out)
+  }
+  else {
+    return(NA)
+  }
 }
 
 #' Search synonyms in pubchem
@@ -375,15 +409,16 @@ pc_prop <- function(cid, properties = NULL, verbose = TRUE, ...) {
 #' @param query character; search term.
 #' @param from character; type of input, can be one of "name" (default), "cid",
 #'     "sid", "aid", "smiles", "inchi", "inchikey"
-#' @param interactive deprecated.  Use the \code{choices} argument instead
-#' @param choices to get only the first synonym, use \code{choices = 1}, to get
-#' a number of synonyms to choose from in an interactive menu, provide the
-#' number of choices you want or "all" to choose from all synonyms.
+#' @param match character; How should multiple hits be handled? \code{"all"}
+#' returns all matches, \code{"first"} returns only the first result,
+#' \code{"ask"} enters an interactive mode and the user is asked for input,
+#' \code{"na"} returns \code{NA} if multiple hits are found.
+#' @param choices deprecated.  Use the \code{match} argument instead.
 #' @param verbose logical; should a verbose output be printed on the console?
-#' @param arg character; optinal arguments like "name_type=word" to match
+#' @param arg character; optional arguments like "name_type=word" to match
 #' individual words.
-#' @param ... optional arguments
-#' @return a list of character vectors (one per query). If \code{choices} is used, a single named vector is returned instead.
+#' @param ... currently unused
+#' @return a named list.
 #'
 #' @references Wang, Y., J. Xiao, T. O. Suzek, et al. 2009 PubChem: A Public
 #' Information System for
@@ -404,50 +439,67 @@ pc_prop <- function(cid, properties = NULL, verbose = TRUE, ...) {
 #' \url{https://pubchemdocs.ncbi.nlm.nih.gov/programmatic-access}, and the data
 #' usage policies of the indicidual data sources
 #' \url{https://pubchem.ncbi.nlm.nih.gov/sources/}.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' \donttest{
 #' pc_synonyms("Aspirin")
 #' pc_synonyms(c("Aspirin", "Triclosan"))
 #' pc_synonyms(5564, from = "cid")
-#' pc_synonyms(c("Aspirin", "Triclosan"), choices = 10)
+#' pc_synonyms(c("Aspirin", "Triclosan"), match = "ask")
 #' }
-pc_synonyms <- function(query, from = "name", choices = NULL, verbose = TRUE,
-                        arg = NULL, interactive = 0, ...) {
+pc_synonyms <- function(query,
+                        from = c("name", "cid", "sid", "aid", "smiles", "inchi", "inchikey"),
+                        match = c("all", "first", "ask", "na"),
+                        verbose = TRUE,
+                        arg = NULL, choices = NULL, ...) {
+
+  if (!ping_service("pc")) stop(webchem_message("service_down"))
+
   # from can be cid | name | smiles | inchi | sdf | inchikey | formula
   # query <- c("Aspirin")
   # from = "name"
-  if (!missing("interactive"))
-    stop("'interactive' is deprecated. Use 'choices' instead.")
+  from <- match.arg(from)
+  match <- match.arg(match)
+  if (!missing("choices"))
+    stop("'choices' is deprecated. Use 'match' instead.")
   foo <- function(query, from, verbose, ...) {
-    if (is.na(query)) return(NA)
+    if (is.na(query)) {
+      if (verbose) webchem_message("na")
+      return(NA)
+    }
     prolog <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug"
     input <- paste0("/compound/", from)
     output <- "/synonyms/JSON"
     if (!is.null(arg))
       arg <- paste0("?", arg)
     qurl <- paste0(prolog, input, output, arg)
-    if (verbose)
-      message(qurl)
+    if (verbose) webchem_message("query", query, appendLF = FALSE)
     Sys.sleep(0.2)
-    cont <- try(content(POST(qurl,
-                             body = paste0(from, "=", query)
-    )), silent = TRUE
-    )
-    if (inherits(cont, "try-error")) {
-      warning("Problem with web service encountered... Returning NA.")
+    res <- try(httr::RETRY("POST",
+                           qurl,
+                           httr::user_agent(webchem_url()),
+                           body = paste0(from, "=", query),
+                           terminate_on = 404,
+                           quiet = TRUE), silent = TRUE)
+    if (inherits(res, "try-error")) {
+      if (verbose) webchem_message("service_down")
       return(NA)
     }
-    if (names(cont) == "Fault") {
-      warning(cont$Fault$Details, ". Returning NA.")
+    if (verbose) message(httr::message_for_status(res))
+    if (res$status_code == 200){
+      cont <- httr::content(res)
+      if (names(cont) == "Fault") {
+        message(cont$Fault$Details, ". Returning NA.")
+        return(NA)
+      }
+      out <- unlist(cont)[-1] #first result is always an ID number
+      names(out) <- NULL
+
+      out <- matcher(out, query = query, match = match, from = from, verbose = verbose)
+    }
+    else {
       return(NA)
     }
-    out <- unlist(cont)
-    names(out) <- NULL
-
-    out <- chooser(out, choices)
-
   }
   out <- lapply(query, foo, from = from, verbose = verbose)
   out <- setNames(out, query)
@@ -491,7 +543,6 @@ pc_synonyms <- function(query, from = "name", choices = NULL, verbose = TRUE,
 #' @references Kim, S., Thiessen, P.A., Cheng, T. et al. PUG-View: programmatic
 #' access to chemical annotations integrated in PubChem. J Cheminform 11, 56
 #' (2019). https://doi.org/10.1186/s13321-019-0375-2.
-#' @author Tamás Stirling, \email{stirling.tamas@@gmail.com}
 #' @seealso \code{\link{get_cid}}, \code{\link{pc_prop}}
 #' @examples
 #' # might fail if API is not available
@@ -502,7 +553,6 @@ pc_synonyms <- function(query, from = "name", choices = NULL, verbose = TRUE,
 #' pc_sect(780286, "modify date", "assay")
 #' pc_sect(9023, "Ensembl ID", "gene")
 #' pc_sect("1ZHY_A", "Sequence", "protein")
-#' pc_sect("US2013040379", "Patent Identifier Synonyms", "patent")
 #' }
 #' @export
 pc_sect <- function(id,
@@ -545,7 +595,6 @@ pc_sect <- function(id,
 #' @references Kim, S., Thiessen, P.A., Cheng, T. et al. PUG-View: programmatic
 #' access to chemical annotations integrated in PubChem. J Cheminform 11, 56
 #' (2019). https://doi.org/10.1186/s13321-019-0375-2.
-#' @author Tamás Stirling, \email{stirling.tamas@@gmail.com}
 #' @examples
 #' # might fail if API is not available
 #' \donttest{
@@ -558,24 +607,31 @@ pc_page <- function(id,
                     domain = c("compound", "substance", "assay", "gene",
                                "protein", "patent"),
                     verbose = TRUE) {
+
+  if (!ping_service("pc")) stop(webchem_message("service_down"))
+
   domain <- match.arg(domain)
   section <- tolower(gsub(" +", "+", section))
   foo <- function(id, section, domain) {
-    qurl <- paste0("https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/data/",
-                   domain, "/", id, "/JSON?heading=", section)
-    if (verbose == TRUE) message("Searching ", id, ". ", appendLF = FALSE)
     if (is.na(id)) {
-      if (verbose == TRUE) {
-        message("Invalid input. Returning NA.")
-      }
+      if (verbose) webchem_message("na")
       return(NA)
     }
+    qurl <- paste0("https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/data/",
+                   domain, "/", id, "/JSON?heading=", section)
+    if (verbose) webchem_message("query", id, appendLF = FALSE)
     Sys.sleep(0.3 + stats::rexp(1, rate = 10 / 0.3))
-    res <- httr::POST(
-      qurl,
-      user_agent("webchem (https://github.com/ropensci/webchem)"))
-    if (res$status_code < 300) {
-      if (verbose == TRUE) message(httr::message_for_status(res))
+    res <- try(httr::RETRY("POST",
+                           qurl,
+                           user_agent(webchem_url()),
+                           terminate_on = 404,
+                           quiet = TRUE), silent = TRUE)
+    if (inherits(res, "try-error")) {
+      if (verbose) webchem_message("service_down")
+      return(NA)
+    }
+    if (verbose) message(httr::message_for_status(res))
+    if (res$status_code == 200) {
       cont <- httr::content(res, type = "text", encoding = "UTF-8")
       cont <- jsonlite::fromJSON(cont, simplifyDataFrame = FALSE)
       tree <- data.tree::as.Node(cont, nameName = "TOCHeading")
@@ -583,9 +639,6 @@ pc_page <- function(id,
       return(tree)
     }
     else {
-      if (verbose == TRUE) {
-        message(paste0(httr::message_for_status(res), " Returning NA."))
-      }
       return(NA)
     }
   }
@@ -621,7 +674,6 @@ pc_page <- function(id,
 #' @references Kim, S., Thiessen, P.A., Cheng, T. et al. PUG-View: programmatic
 #' access to chemical annotations integrated in PubChem. J Cheminform 11, 56
 #' (2019). https://doi.org/10.1186/s13321-019-0375-2.
-#' @author Tamás Stirling, \email{stirling.tamas@@gmail.com}
 #' @examples
 #' # might fail if API is not available
 #' \donttest{
