@@ -22,7 +22,6 @@
 #' Ralf B. Schäfer (2020). webchem: An R Package to Retrieve Chemical
 #' Information from the Web. Journal of Statistical Software, 93(13).
 #' <doi:10.18637/jss.v093.i13>.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' is.inchikey('BQJCRHHNABKAKU-KBQPJGBKSA-N')
@@ -53,7 +52,6 @@ is.inchikey = function(x, type = c('format', 'chemspider'), verbose = TRUE) {
 #' @return a logical
 #'
 #' @seealso \code{\link{is.inchikey}} for a pure-R implementation.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' \donttest{
@@ -67,21 +65,38 @@ is.inchikey = function(x, type = c('format', 'chemspider'), verbose = TRUE) {
 #' is.inchikey_cs('BQJCRHHNABKAKU-KBQPJGBKSB-N')
 #' }
 is.inchikey_cs <- function(x, verbose = TRUE){
-  # x <- 'BQJCRHHNABKAKU-KBQPJGBKSA'
+
+  if (!ping_service("cs_web")) stop(webchem_message("service_down"))
+
   if (length(x) > 1) {
     stop('Cannot handle multiple input strings.')
   }
+  if (is.na(x)) {
+    if (verbose) webchem_message("na")
+    return(NA)
+  }
   baseurl <- 'http://www.chemspider.com/InChI.asmx/IsValidInChIKey?'
   qurl <- paste0(baseurl, 'inchi_key=', x)
   Sys.sleep(0.1)
-  h <- try(read_xml(qurl), silent = TRUE)
-  if (inherits(h, "try-error")) {
-    warning('Problem with webservice... Returning NA.')
-    out <- NA
-  } else {
+  if (verbose) webchem_message("query", x, appendLF = FALSE)
+  res <- try(httr::RETRY("GET",
+                         qurl,
+                         httr::user_agent(webchem_url()),
+                         terminate_on = 404,
+                         quiet = TRUE), silent = TRUE)
+  if (inherits(res, "try-error")) {
+    if (verbose) webchem_message("service_down")
+    return(NA)
+  }
+  if (verbose) message(httr::message_for_status(res))
+  if (res$status_code == 200){
+    h <- xml2::read_xml(res)
     out <- as.logical(xml_text(h))
+    return(out)
+    }
+  else {
+    return(NA)
   }
-  return(out)
 }
 
 
@@ -101,7 +116,6 @@ is.inchikey_cs <- function(x, verbose = TRUE){
 #' @return a logical
 #'
 #' @seealso \code{\link{is.inchikey}} for a pure-R implementation.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' \donttest{
@@ -177,7 +191,6 @@ is.inchikey_format = function(x, verbose = TRUE) {
 #' Ralf B. Schäfer (2020). webchem: An R Package to Retrieve Chemical
 #' Information from the Web. Journal of Statistical Software, 93(13).
 #' <doi:10.18637/jss.v093.i13>.
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #'
 #' @export
 #' @examples
@@ -188,60 +201,62 @@ is.inchikey_format = function(x, verbose = TRUE) {
 #' is.cas('64-17-55')
 #' is.cas('64-17-6')
 is.cas <-  function(x, verbose = TRUE) {
-  # x <- '64-17-5'
-  if (length(x) > 1) {
-    stop('Cannot handle multiple input strings.')
-  }
 
-  # cas must not have any alpha characters
-  if(grepl(pattern = "[[:alpha:]]", x = x)){
-    if(isTRUE(verbose)){message("String contains alpha characters")}
-    return(FALSE)
-  }
+  foo <- function(x, verbose) {
+    # pass NA's through
+    if (is.na(x)) return(NA)
+    # cas must not have any alpha characters
+    if (grepl(pattern = "[[:alpha:]]", x = x)) {
+      if (isTRUE(verbose)) {
+        message(x,": String contains alpha characters")
+        }
+      return(FALSE)
+    }
 
-  # cas must have two hyphens
-  nsep <- str_count(x, '-')
-  if (nsep != 2) {
-    if (isTRUE(verbose))
-      message('Less than 2 hyphens in string.')
-    return(FALSE)
-  }
+    # cas must have two hyphens
+    nsep <- str_count(x, '-')
+    if (nsep != 2) {
+      if (isTRUE(verbose))
+        message(x, ': Less than 2 hyphens in string.')
+      return(FALSE)
+    }
 
-  # first part 2 to 7 digits
-  fi <- gsub('^(.*)-(.*)-(.*)$', '\\1', x)
-  if (nchar(fi) > 7 | nchar(fi) < 2) {
-    if (isTRUE(verbose))
-      message('First part with more than 7 digits!')
-    return(FALSE)
-  }
+    # first part 2 to 7 digits
+    fi <- gsub('^(.*)-(.*)-(.*)$', '\\1', x)
+    if (nchar(fi) > 7 | nchar(fi) < 2) {
+      if (isTRUE(verbose))
+        message(x, ': First part has more than 7 digits!')
+      return(FALSE)
+    }
 
-  # second part must be two digits
-  se <- gsub('^(.*)-(.*)-(.*)$', '\\2', x)
-  if (nchar(se) != 2) {
-    if (isTRUE(verbose))
-      message('Second part has not two digits!')
-    return(FALSE)
-  }
+    # second part must be two digits
+    se <- gsub('^(.*)-(.*)-(.*)$', '\\2', x)
+    if (nchar(se) != 2) {
+      if (isTRUE(verbose))
+        message(x, ': Second part should have two digits!')
+      return(FALSE)
+    }
 
-  # third part (checksum) must be 1 digit
-  th <- gsub('^(.*)-(.*)-(.*)$', '\\3', x)
-  if (nchar(th) != 1) {
-    if (isTRUE(verbose))
-      message('Third part has not 1 digit!')
-    return(FALSE)
-  }
+    # third part (checksum) must be 1 digit
+    th <- gsub('^(.*)-(.*)-(.*)$', '\\3', x)
+    if (nchar(th) != 1) {
+      if (isTRUE(verbose))
+        message(x, ': Third part should have 1 digit!')
+      return(FALSE)
+    }
 
-  # check checksum
-  di <-  as.numeric(strsplit(gsub('^(.*)-(.*)-(.*)$', '\\1\\2', x),
-                             split = '')[[1]])
-  checksum <- sum(rev(seq_along(di)) * di)
-  if (checksum %% 10 != as.numeric(th)) {
-    if (isTRUE(verbose))
-      message('Checksum is not correct! ', checksum %% 10, ' vs. ', th)
-    return(FALSE)
+    # check checksum
+    di <-  as.numeric(strsplit(gsub('^(.*)-(.*)-(.*)$', '\\1\\2', x),
+                               split = '')[[1]])
+    checksum <- sum(rev(seq_along(di)) * di)
+    if (checksum %% 10 != as.numeric(th)) {
+      if (isTRUE(verbose))
+        message(x, ': Checksum is not correct! ', checksum %% 10, ' vs. ', th)
+      return(FALSE)
+    }
+    return(TRUE)
   }
-
-  return(TRUE)
+  return(sapply(x, foo, verbose))
 }
 
 
@@ -256,8 +271,6 @@ is.cas <-  function(x, verbose = TRUE) {
 #'
 #' @note This function can handle only one SMILES string.
 #'
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
-#'
 #' @references Egon Willighagen (2015). How to test SMILES strings in
 #' Supplementary Information.
 #' \url{https://chem-bla-ics.blogspot.nl/2015/10/how-to-test-smiles-strings-in.html}
@@ -320,11 +333,10 @@ extr_num <- function(x) {
 #' 1 = first atom, 2 = second atom, t = bond type, s = stereo type, x = not
 #' used, r = bond typology, c = reacting center status.
 #'
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @references Grabner, M., Varmuza, K., & Dehmer, M. (2012). RMol:
 #' a toolset for transforming SD/Molfile structure information into R objects.
 #' Source Code for Biology and Medicine, 7, 12.
-#' \url{http://doi.org/10.1186/1751-0473-7-12}
+#' \url{https://doi.org/10.1186/1751-0473-7-12}
 #' @export
 
 parse_mol <- function(string) {
@@ -364,7 +376,6 @@ parse_mol <- function(string) {
 #' @return character vector of valid CAS numbers
 #' @seealso \code{\link{is.cas}}
 #' @export
-#' @author Eric Scott, \email{scottericr@@gmail.com}
 #' @examples
 #' x = c(58082, 123456, "hexenol")
 #' as.cas(x)
@@ -387,59 +398,17 @@ as.cas <- function(x){
 }
 
 
-#' Interactively choose a result from a menu
-#' @description In interactive sessions, prompts a user to choose an element of
-#' a vector from a menu. Use this for all functions that return multiple
-#' possible results such as multiple identifiers or synonyms.
-#' @param x a character vector
-#' @param choices If \code{choices = "all"} then the entire vector \code{x} is
-#' used for the menu.  If numeric > 1, only that number of elements from the
-#' start of \code{x} are shown. If \code{choices = 1}, then the first element of
-#' \code{x} is returned without prompting the user.  If \code{NULL} then
-#' \code{x} is returned unchanged.
-#'
-#' @importFrom utils menu
-#' @importFrom utils head
-#' @return a character vector of length 1
-#' @noRd
-#'
-#' @examples
-#' test <- c("apples", "bananas", "orange", "plum", "peach", "guava", "kumquat")
-#' chooser(test, "all")
-#' chooser(test, 3)
-chooser <- function(x, choices){
-  if(interactive() & !is.null(choices)){
-    #only in an interactive R session when number of choices is specified
-    if(is.numeric(choices) & choices > length(x)) {
-      choices = "all"
-      warning('Number of choices excedes length of x, using all choices instead',
-              immediate. = TRUE)
-    }
-    if(choices == "all") { #then give all of x as possible choices
-      pick <- menu(x, graphics = FALSE, 'Select one:')
-      out <- x[pick]
-    }
-    if(choices == 1) {
-      out <- x[1]
-    }
-    if(is.numeric(choices) & choices > 1){
-      pick <- menu(head(x, choices), graphics = FALSE, 'Select one:')
-      out <- x[pick]
-    }
-  } else {
-    out <- x
-  }
-  return(out)
-}
 
-#' matcher utility
+#' Used internally to handle the `match` argument in most functions.
 #'
-#' @param x a vector
+#' @param x a vector of hits returned from a query
 #' @param query what the query was, only used if match = "best"
-#' @param result what the result of the query was, only used if match = "best
-#' @param match haracter; How should multiple hits be handeled? "all" returns
-#' all matched IDs, "first" only the first match, "best" the best matching (by
-#' name) ID, "ask" is a interactive mode and the user is asked for input, "na"
+#' @param result vector of results of the same type as `query` and same length
+#'   as `x`, only used if match = "best
+#' @param match character; How should multiple hits be handled? "all" returns
+#'   all matched IDs, "first" only the first match, "best" the best matching (by
+#'   name) ID, "ask" is a interactive mode and the user is asked for input, "na"
+#' @param from character; used only to check that match = "best" is used sensibly.
 #' @param verbose print messages?
 #'
 #' @return
@@ -454,36 +423,38 @@ matcher <-
            query = NULL,
            result = NULL,
            match = c("all", "best", "first", "ask", "na"),
+           from = NULL,
            verbose = FALSE) {
 
     match <- match.arg(match)
     names(x) <- result
 
-    if(length(x) == 1) {
+    if (length(x) == 1) {
       return(x)
     } else {
-      if(verbose) {
-        message("More then one Link found for '", query, "'. \n")
+      if (verbose) message(" Multiple found. ", appendLF = FALSE)
+
+      if (!is.null(from)) {
+        if (!str_detect(tolower(from), "name") & match == "best") {
+          warning("match = 'best' only makes sense for chemical name queries.\n Setting match = 'first'.")
+          match <- "first"
+        }
       }
 
-      if(match == "all") {
-        if(verbose) {
-          message("Returning all matches. \n")
-        }
+      if (match == "all") {
+        if (verbose) message("Returning all.")
         return(x)
+      }
 
-      } else if (match == "best") {
+      else if (match == "best") {
         #check that x and result are same length
-        if(length(x) != length(result))
+        if (length(x) != length(result))
           stop("Can't use match = 'best' without query matches for each output")
-        if (verbose) {
-          message("Returning best match. \n")
-        }
+        if (verbose) message("Returning best.")
         dd <- adist(query, result) / nchar(result)
         return(x[which.min(dd)])
       } else if (match == "first") {
-        if (verbose)
-          message("Returning first match. \n")
+        if (verbose) message("Returning first.")
         return(x[1])
 
       } else if (match == "ask" & interactive()) {
@@ -496,12 +467,45 @@ matcher <-
         return(x[pick])
 
       } else if (match == "na") {
-        if (verbose) {
-          message("Returning NA. \n")
-        }
+        if (verbose) message("Returning NA.")
         x <- NA
-        names(x)<-NA
+        names(x) <- NA
         return(x)
       }
     }
   }
+
+#' Webchem messages
+#'
+#' Webchem spacific messages to be used in verbose messages.
+#' @noRd
+webchem_message <- function(action = c("na",
+                                       "query",
+                                       "query_all",
+                                       "not_found",
+                                       "not_available",
+                                       "service_down"),
+                            appendLF = TRUE,
+                            ...) {
+  action <- match.arg(action)
+  string <- switch(
+    action,
+    na = "Query is NA. Returning NA.",
+    query = paste0("Querying ", ..., ". "),
+    query_all = "Querying. ",
+    not_found = " Not found. Returning NA.",
+    not_available = " Not available. Returning NA.",
+    service_down = " Service not available. Returning NA."
+    )
+  message(string, appendLF = FALSE)
+  if (appendLF) message("")
+}
+
+#' Webchem URL
+#'
+#' URL of the webchem package to be used in httr::user_agent()
+#' @noRd
+webchem_url <- function() {
+  url <- "https://cran.r-project.org/web/packages/webchem/index.html"
+  return(url)
+}
