@@ -17,8 +17,6 @@
 #'
 #' @note Only matches in labels are returned.
 #'
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
-#'
 #' @import jsonlite httr
 #' @importFrom stats rgamma
 #' @importFrom utils URLencode URLdecode
@@ -40,58 +38,68 @@ get_wdid <-
            verbose = TRUE,
            language = 'en') {
 
+    if (!ping_service("wd")) stop(webchem_message("service_down"))
+
   # language <-  'en'
   # query <- 'Triclosan'
 
   match <- match.arg(match)
   foo <- function(query, language, match, verbose){
     if (is.na(query)){
+      if (verbose) webchem_message("na")
       id <- NA
       matched_sub <- NA
     } else {
-      query1 <- URLencode(query)
+      query1 <- URLencode(query, reserved = TRUE)
       limit <- 50
       qurl <-
         paste0("wikidata.org/w/api.php?action=wbsearchentities&format=json&type=item")
       qurl <- paste0(qurl, "&language=", language, "&limit=", limit, "&search=", query1)
-      if (verbose)
-        message('Querying ', qurl)
+      if (verbose) webchem_message("query", query, appendLF = FALSE)
       Sys.sleep(0.3)
-      cont <-
-        fromJSON(content(GET(
-          qurl,
-          user_agent('webchem (https://github.com/ropensci/webchem)')
-        ), 'text'))
-      search <- cont$search
-      if (length(search) == 0) {
-        if (verbose)
-          message('Substance not found! Returing NA. \n')
-        id <- NA
-        matched_sub <- NA
-      } else {
-        # use only matches on label
-        search <- search[search$match$type %in% c('label', 'alias'), ]
-        # # check matches
-        search <- search[tolower(iconv(search$match$text,
-                                       "latin1",
-                                       "ASCII",
-                                       sub = "")) == tolower(query), ]
-
-        if (nrow(search) > 1) {
+      res <- try(httr::RETRY("GET",
+                             qurl,
+                             httr::user_agent(webchem_url()),
+                             terminate_on = 404,
+                             quiet = TRUE), silent = TRUE)
+      if (inherits(res, "try-error")) {
+        if (verbose) webchem_message("service_down")
+        return(tibble::tibble(query = query, match = NA, wdid = NA))
+      }
+      if (verbose) message(httr::message_for_status(res))
+      if (res$status_code == 200) {
+        cont <- jsonlite::fromJSON(httr::content(res,
+                                                 type = "text",
+                                                 encoding = "utf-8"))
+        search <- cont$search
+        if (length(search) == 0) {
+          if (verbose) webchem_message("not_found")
+          id <- NA
+          matched_sub <- NA
+        } else {
+          # use only matches on label
+          search <- search[search$match$type %in% c('label', 'alias'), ]
+          # # check matches
+          search <- search[tolower(iconv(search$match$text,
+                                         "latin1",
+                                         "ASCII",
+                                         sub = "")) == tolower(query), ]
           id <-
             matcher(
               search$id,
               query = query,
               result = search$label,
               match = match,
+              # from = from,
               verbose = verbose
             )
           matched_sub <- names(id)
-        } else {
-          id <- search$id
-          matched_sub <- search$label
         }
       }
+      else {
+        id <- NA
+        matched_sub <- NA
+      }
     }
     out <- tibble(query = query, match = matched_sub, wdid = id)
     return(out)
@@ -134,7 +142,6 @@ get_wdid <-
 #' Mitraka, Elvira, Andra Waagmeester, Sebastian Burgstaller-Muehlbacher, et al. 2015
 #' Wikidata: A Platform for Data Integration and Dissemination for the Life Sciences and beyond. bioRxiv: 031971.
 #'
-#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' \dontrun{
@@ -142,16 +149,19 @@ get_wdid <-
 #'  wd_ident(id)
 #' }
 wd_ident <- function(id, verbose = TRUE){
+
+  if (!ping_service("wd")) stop(webchem_message("service_down"))
+
   # id <- c( "Q163648", "Q18216")
   # id <- 'Q408646'
   foo <- function(id, verbose){
+    empty <- as.list(rep(NA, 13))
+    names(empty) <- c("smiles", "cas", "cid", "einecs", "csid", "inchi",
+                      "inchikey", "drugbank", "zvg", "chebi", "chembl", "unii",
+                      "source_url")
     if (is.na(id)) {
-      if (verbose)
-        message('NA as input! Returing NA. \n')
-      out <- as.list(rep(NA, 13))
-      names(out) <- c("smiles", "cas", "cid", "einecs", "csid", "inchi", "inchikey",
-                      "drugbank", "zvg", "chebi", "chembl", "unii", "source_url")
-      return(out)
+      if (verbose) webchem_message("na")
+      return(empty)
     }
     baseurl <- 'https://query.wikidata.org/sparql?format=json&query='
     props <- c('P233', 'P231', 'P662', 'P232', 'P661', 'P234', 'P235', 'P715', 'P679',
@@ -168,38 +178,50 @@ wd_ident <- function(id, verbose = TRUE){
     qurl <- paste0(baseurl, sparql)
     qurl <- URLencode(qurl)
     Sys.sleep( rgamma(1, shape = 15, scale = 1/10))
-    if (verbose)
-      message('Querying ', qurl)
-    res <- GET(qurl)
-    tmp <- fromJSON(content(res, as = "text"))
+    if (verbose) webchem_message("query", id, appendLF = FALSE)
+    res <- try(httr::RETRY("GET",
+                           qurl,
+                           httr::user_agent(webchem_url()),
+                           terminate_on = 404,
+                           quiet = TRUE), silent = TRUE)
+    if (inherits(res, "try-error")) {
+      if (verbose) webchem_message("service_down")
+      return(empty)
+    }
+    if (verbose) message(httr::message_for_status(res))
+    if (res$status_code == 200) {
+      tmp <- fromJSON(content(res, as = "text"))
 
-    vars_out <- tmp$head$vars
-    out <- tmp$results$bindings
+      vars_out <- tmp$head$vars
+      out <- tmp$results$bindings
 
-    if (length(out) == 0) {
-      if (verbose)
-        message('Not found! Returing NA. \n')
-      out <- as.list(rep(NA, 13))
-      names(out) <- c(vars_out, 'source_url')
+      if (length(out) == 0) {
+        if (verbose) webchem_message("not_found")
+        out <- as.list(rep(NA, 13))
+        names(out) <- c(vars_out, 'source_url')
+        return(out)
+      }
+
+      if (nrow(out) > 1) {
+        message("More then one unique entry found! Returning first.")
+        out <- out[1, ]
+      }
+
+      out <- lapply(out, '[[', 'value')
+
+      # check for missing entries and add to out-list
+      miss <- names[!names %in% names(out)]
+      for (i in miss) {
+        out[[i]] <- NA
+      }
+      out <- out[names]
+      out[['source_url']] <- paste0('https://www.wikidata.org/wiki/', id)
+      out <- unlist(out)
       return(out)
     }
-
-    if (nrow(out) > 1) {
-      warning("More then one unique entry found! Returning first.")
-      out <- out[1, ]
+    else {
+      return(empty)
     }
-
-    out <- lapply(out, '[[', 'value')
-
-    # check for missing entries and add to out-list
-    miss <- names[!names %in% names(out)]
-    for (i in miss) {
-      out[[i]] <- NA
-    }
-    out <- out[names]
-    out[['source_url']] <- paste0('https://www.wikidata.org/wiki/', id)
-    out <- unlist(out)
-    return(out)
   }
   # ugly fixing to return data.frame
   out <- data.frame(t(sapply(id, foo,verbose = verbose)), stringsAsFactors = FALSE, row.names = seq_along(id))
